#!/usr/bin/perl -w

# Copyright (c) 2003 Jeffrey E. Barrick
# All rights reserved.  Redistribution is not permitted without the 
# express written permission of the authors.
#

use Bio::Root::Root;

package Intergenic::Stockholm;
use vars qw(@ISA);
use strict;

@ISA = qw( Bio::Root::Root );

use Data::Dumper;


our $_open_pairs = '\<\[\{\(';
our $_close_pairs = '\>\]\}\)';
our $_not_gaps = "ATCGU";

=head2 new

 Title   : new
 Usage   : $dbcache = Intergenic::Cache->new( -cache_name => 'cog' );
 Function: Creates/opens a database to cache values to
 Returns : a new DatabaseCache object

=cut

sub new
{
	my($caller,@args) = @_;
	my $class = ref($caller) || $caller;

	my $self = Bio::Root::Root->new($caller, @args);

	
	$self->{file_name} = $self->Bio::Root::RootI::_rearrange([qw(FILENAME)], @args);
	$self->{file_name} = $self->Bio::Root::RootI::_rearrange([qw(IN)], @args) if (!defined $self->{file_name});
	bless ($self, $class);
	
	#Initialize
	$self->{stockholm}->{seq} = ();
	
	#Drop if no file name (which means to load);
	return $self if (!$self->{file_name});
	
	$self->readfile($self->{file_name});
	return $self;
}

=head2 readfile

 Title   : readfile
 Usage   : Intergenic::Cache->readfile( $filename );
 Function: Reads Stockholm alignment from file
 Returns : nothing

=cut

sub readfile
{
	my ($self, $filename) = @_;
	
	$self->{stockholm} = {};
	$self->{stockholm}->{seq} = [];

	open INFILE, "<$filename" or die "Could not open $filename";

	my $start = 0;
	my @gfList=();
	LINE: while (<INFILE>)
	{
		chomp $_;
		
		#Should recognize what version number it is from first line...
	
		#print "$_\n";
		next LINE if ($_ =~ m/^\s*$/);
		last LINE if ($_ =~ m/\/\//);
		
		
		#From documentation in INFERNAL 0.55 manual
		
		#Per-file annotation
		if ($_ =~ s/^#=GF\s+//)
		{
		    push @gfList,$_;
		}
		#Per sequence annotation
		elsif ($_ =~ s/^#=GS\s+//)
		{
			$_ =~ m/(\S+)\s+(\S+)\s+(.*)/;
			my ($name, $tag, $data) = ($1, $2, $3);
			my $id = $self->{stockholm}->{seq_hash}->{$name};
			if (not defined $id)
			{
				$id = scalar @{$self->{stockholm}->{seq}};
				$self->{stockholm}->{seq_hash}->{$name} = $id;
				$self->{stockholm}->{seq}->[$id]->{name} = $name;
			}
			$self->{stockholm}->{seq}->[$id]->{GS}->{$tag} = $data;
			next LINE;
		}
		#Per-column annotation
		elsif ($_ =~ s/^#=GC\s+//)
		{
			my ($tag, $data) = split /\s+/, $_;
			$self->{stockholm}->{GC}->{"$tag"} .= $data;
			next LINE;
		}
		#Per-residue annotation
		elsif ($_ =~ s/^#=GR\s+//)
		{
			my ($name, $tag, $data) = split /\s+/, $_;
						
			#Create sequence if it doesn't exist
			my $id = $self->{stockholm}->{seq_hash}->{$name};
			if (not defined $id)
			{
				$id = scalar @{$self->{stockholm}->{seq}};
				$self->{stockholm}->{seq_hash}->{$name} = $id;
				$self->{stockholm}->{seq}->[$id]->{name} = $name;
			}
			$self->{stockholm}->{seq}->[$id]->{GR}->{"$tag"} .= $data;
			next LINE;
		}
		#Sequence
		elsif ( !($_ =~ m/^#/) ) 
		{
			#Split into name and sequence
			my ($name, $seq) = split /\s+/, $_;
			
			my $id = $self->{stockholm}->{seq_hash}->{$name};
			if (not defined $id)
			{
				$id = scalar @{$self->{stockholm}->{seq}};
				$self->{stockholm}->{seq_hash}->{$name} = $id;
				$self->{stockholm}->{seq}->[$id]->{name} = $name;
			}
			
			$self->{stockholm}->{seq}->[$id]->{seq} .= $seq;
		}
	
	}
	
	close INFILE;

	my $gf;
	for $gf (@gfList) {
	    $gf  =~ m/(\S+)\s+(.*)/;
	    my ($tag, $data) = ($1, $2);
	    $self->{stockholm}->{GF}->{"$tag"} = $data;
	}

	#print Dumper($self->{stockholm});
}

=head2 writefile

 Title   : writefile
 Usage   : Intergenic::Stockholm->writefile( $filename );
 Function: Writes alignment in Stockholm format
 Returns : nothing

=cut

sub writefile
{
	my ($self, $filename) = @_;
	
	open OUTFILE, ">$filename" or die "Could not open $filename";
	print OUTFILE "# STOCKHOLM 1.0\n\n";
	
	my $begin_padding = 2;


	#Header
	
	#Per-file annotation
	foreach my $key ( sort keys %{$self->{stockholm}->{GF}} )
	{
		print OUTFILE "#=GF $key $self->{stockholm}->{GF}->{$key}\n";
	}
	print OUTFILE "\n";

	# #=GS per-sequence annotation lines	
	#Make a list of keys to go through in order;
	my $gs_keys;
	for (my $i = 0; $i < scalar @{$self->{stockholm}->{seq}}; $i++)
	{
		foreach my $key (sort keys %{$self->{stockholm}->{seq}->[$i]->{GS}})
		{
			$gs_keys->{$key}++;
		}
	}
	
	#Figure out the lengths of the line labels
	
	my $begin_line_length = 0;
	my $gr_tag_length = 0;
	my $tag_length;

	#Test length of #=GS lines
	$tag_length = length "#=GS ";
	for my $key (sort keys %$gs_keys)
	{		
			my $test_len = $tag_length + length($key);			
			$gr_tag_length = length($key) if (length($key) > $gr_tag_length);
	}

#	
#	#Test length of #=GS lines
#	$tag_length = length "#=GS ";
#	for my $key (sort keys %$gs_keys)
#	{		
#		for (my $i = 0; $i < scalar @{$self->{stockholm}->{seq}}; $i++)
#		{
#			next if (!defined $self->{stockholm}->{seq}->[$i]->{GS}->{$key});
#			my $test_len = $tag_length + length($self->{stockholm}->{seq}->[$i]->{GS}->{$key});
#			$begin_line_length = $test_len if ($test_len >  $begin_line_length);
#			$gr_tag_length = length($key) if (length($key) > $gr_tag_length);
#		}
#	}
	
	
	#Test lengths of sequence name lines
	for (my $i = 0; $i < scalar @{$self->{stockholm}->{seq}}; $i++)
	{
		my $test_len = length($self->{stockholm}->{seq}->[$i]->{name});
		$begin_line_length = $test_len if ($test_len >  $begin_line_length);
	}
	
	my $name_max_length = $begin_line_length;
	
	#Test lengths of #=GR lines
	$tag_length = length "#=GR ";
	for (my $i = 0; $i < scalar @{$self->{stockholm}->{seq}}; $i++)
	{
		foreach my $key (sort keys %{$self->{stockholm}->{seq}->[$i]->{GR}} )
		{
			my $test_len = $tag_length + length("$self->{stockholm}->{seq}->[$i]->{name} $key");
			$begin_line_length = $test_len if ($test_len >  $begin_line_length);
			$gr_tag_length = length($key) if (length($key) > $gr_tag_length);
		}
	}
	#Test lengths of #=GC lines
	$tag_length = length "#=GC ";
	foreach my $key (sort keys %{$self->{stockholm}->{GC}})
	{
		my $test_len = $tag_length + length($key);
		$begin_line_length = $test_len if ($test_len >  $begin_line_length);
	}
	
	$begin_line_length += $begin_padding; #$begin_padding on the right side
	
	#Header GS lines
	
	#Write each GS key as a separate chunk
	for my $key (sort keys %$gs_keys)
	{
		for (my $i = 0; $i < scalar @{$self->{stockholm}->{seq}}; $i++)
		{
			next if (!defined $self->{stockholm}->{seq}->[$i]->{GS}->{$key});
			
			print OUTFILE "#=GS $self->{stockholm}->{seq}->[$i]->{name}";
			print OUTFILE " " x ($name_max_length - length("$self->{stockholm}->{seq}->[$i]->{name}")) . "  ";
			print OUTFILE "$key" . " " x ($gr_tag_length - length($key)) . "  " . $self->{stockholm}->{seq}->[$i]->{GS}->{$key} . "\n";
		}
		print OUTFILE "\n";
	}
	
	#Main alignment
	for (my $i = 0; $i < scalar @{$self->{stockholm}->{seq}}; $i++)
	{
		print OUTFILE $self->{stockholm}->{seq}->[$i]->{name} . " " x ($begin_line_length - length($self->{stockholm}->{seq}->[$i]->{name}) - 1) . " " . $self->{stockholm}->{seq}->[$i]->{seq} . "\n";
	
		foreach my $key (sort keys %{$self->{stockholm}->{seq}->[$i]->{GR}})
		{
			my $temp_len = $begin_line_length;
			print OUTFILE "#=GR $self->{stockholm}->{seq}->[$i]->{name}";
			$temp_len -= length("#=GR $self->{stockholm}->{seq}->[$i]->{name}");
			my $pad_len = $temp_len  - $gr_tag_length - $begin_padding;
			print OUTFILE " " x ($pad_len - 1) . " ";
			$temp_len -= $pad_len;
			print OUTFILE "$key" . " " x ($gr_tag_length - length($key) + $begin_padding) . $self->{stockholm}->{seq}->[$i]->{GR}->{$key} . "\n";
		}
	}
	
	foreach my $key (sort keys %{$self->{stockholm}->{GC}})
	{
		print OUTFILE "#=GC $key" . " " x ($begin_line_length - length("#=GC $key")) . $self->{stockholm}->{GC}->{$key} . "\n";
	}
	
	print OUTFILE "\n//\n";
	close OUTFILE;

}

=head2 paired_columns

 Title   : paired_columns
 Usage   : $pair_list = Intergenic::Stockholm->paired_columns( $gc_key );
 Function: $gc_key is optional and defualts to pairs in SS_cons line
 Returns : Reference to a list of column pairs in the given GC line
           each pair has an {open} and {close} key to the respective column number
           (starting at zero).
=cut


sub paired_columns
{
	my ($self, $gc_key) = @_;
	$gc_key = 'SS_cons' if (!$gc_key);
	
	#collect pairs by reading up and down mountain
	my @pairs;
	my @open_pair_positions;
	for (my $i = 0; $i < length $self->{stockholm}->{GC}->{$gc_key}; $i++)
	{
		my $c = substr $self->{stockholm}->{GC}->{$gc_key}, $i, 1;
		if ( $_open_pairs =~ m/\Q$c/ )
		{
			push @open_pair_positions, $i;
		}
		elsif ( $_close_pairs =~ m/\Q$c/ )
		{
			my $j = pop @open_pair_positions;
			die "There are too many closing base pairs." if (!defined $j); 
			my $pair = { 'open' =>  $j, 'close' => $i };
			push @pairs, $pair;
		}
	}
	#Final check to see if there are too many opening base pairs.
	die "There are too many opening base pairs." if (scalar @open_pair_positions > 0); 
	
	return \@pairs;
}

=head2 column_paired_array

 Title   : column_paired_array
 Usage   : $pair_list = Intergenic::Stockholm->column_paired_array( $gc_key );
 Function: $gc_key is optional and defualts to pairs in SS_cons line
 Returns : List of 0 or 1 for each column of the alignment indicating whether that
 			column is in a pair.
=cut


sub column_paired_array
{
	my ($self, $gc_key) = @_;
	$gc_key = 'SS_cons' if (!$gc_key);
	
	#collect pairs by reading up and down mountain
	my @pairs;
	my @open_pair_positions;
	for (my $i = 0; $i < length $self->{stockholm}->{GC}->{$gc_key}; $i++)
	{
		$pairs[$i] = 0;
		my $c = substr $self->{stockholm}->{GC}->{$gc_key}, $i, 1;
		if (( $_open_pairs =~ m/\Q$c/ ) || ( $_close_pairs =~ m/\Q$c/ ))
		{
			$pairs[$i] = 1;
		}
	}
	
	return @pairs;
}

=head2 ss_cons

 Title   : get_ss
 Usage   : Intergenic::Stockholm->ss_cons( );
 Function: get/set the secondary structure consensus
 Returns : 
 
=cut

sub ss_cons
{
 	my ($self, $ss_cons) = @_;
 	$self->{stockholm}->{GC}->{SS_cons} = $ss_cons if (defined $ss_cons);
 	return $self->{stockholm}->{GC}->{SS_cons};
}


=head2 sort

 Title   : sort
 Usage   : Intergenic::Stockholm->sort( );
 Function: Sorts the alignment entries by accession/start-end
 Returns : 
 
=cut

sub sort_by_accession
{
 	my ($self)
}

=head2 remove_redundant

 Title   : remove_redundant
 Usage   : Intergenic::Stockholm->remove_redundant( $identity );
 Function: Removes sequences that have more than this fraction identity, counting gaps
           keeps the one of a pair with a lower accession number.
 Returns : 
 
=cut

sub remove_redundant
{
	my ($self, $identity, $strict, $generate_comments) = @_;
	$identity = 1 if (!defined $identity);
	$generate_comments = 0 if (!defined $generate_comments);
	
	my $sto = $self->{stockholm};
	
	my $rf_len = $self->alignment_length;
	my @rf_list;
	if (!$strict)
	{
		$self->throw("Alignment must have an RF line.") if (!$sto->{GC}->{RF});
		my $rf = "\U$sto->{GC}->{RF}";
		@rf_list = split //,  $rf;
		foreach (my $i = 0; $i < scalar @rf_list; $i++)
		{
			$rf_list[$i] = 0 if (! ($rf_list[$i]=~ m/[ATCGU]/) );
		}
		
		$rf =~ s/[^ATCGU]//g;
		$rf_len = length $rf;
	}
	
	COMPARE: foreach (my $i = 0; $i < scalar @{$sto->{seq}}; $i++)
	{
		my $on_seq = $sto->{seq}->[$i]->{seq};
		
		foreach (my $j = 0; $j < $i; $j++)
		{
			my $test_seq = $sto->{seq}->[$j]->{seq};
				
			my $pi = 0;
			foreach (my $q = 0; $q < length $test_seq; $q++)
			{
				next if (!$strict && !$rf_list[$q]);
				$pi++ if ( (substr $on_seq, $q, 1) eq (substr $test_seq, $q, 1) );
			}
			$pi /= $rf_len;
			
			#print "$pi $sto->{seq}->[$i]->{name} $sto->{seq}->[$j]->{name}\n";
			
			if ($pi >= $identity)
			{
				if ($generate_comments)
				{
					$self->{stockholm}->{GF}->{"REDUNDANT $sto->{seq}->[$i]->{name}"} = "$sto->{seq}->[$j]->{name}";
				}
			
				delete $sto->{seq_hash}->{$sto->{seq}->[$i]->{name}};
				splice @{$sto->{seq}}, $i, 1;
				$i--;
				next COMPARE;
			}
		}
	}
	
	$self->delete_gapped_columns;# if (!$strict);
}

=head2 prune_to_most_diverse

 Title   : prune_to_most_diverse
 Usage   : Intergenic::Stockholm->prune_to_most_diverse( $identity );
 Function: Groups sequences into the requested number of subsets, then keeps the
           most diverse set of sequences.
 Returns : 1 if pruning was necessary
 
=cut

sub prune_to_most_diverse
{
	my ($self, $num, $strict) = @_;
	$self->throw("You must request the number of sequences to keep.") if (!$num);

	my $sto = $self->{stockholm};

	#The group list indexes each sequence to a group
	my @group_list;
	foreach (my $i = 0; $i < scalar $self->seq; $i++)
	{
		push @{$group_list[$i]}, $i;
	}
	
	return 0 if (scalar @group_list <= $num); #We already have fewer than this many sequences...

	#Calculate the score matrix
	my @matrix = $self->similarity_matrix($strict);
	#print Dumper(@matrix);

	#Group most similar
	while (scalar @group_list > $num)
	{
		my $best_merge;
		foreach (my $i = 0; $i < scalar @group_list; $i++)
		{		
			foreach (my $j = 0; $j < $i; $j++)
			{
				my $avg_similarity = 0;
				
				foreach my $group_i (@{$group_list[$i]})
				{
					foreach my $group_j (@{$group_list[$j]})
					{
						$avg_similarity += $matrix[$group_i][$group_j];
					}
				}
				$avg_similarity /= scalar (@{$group_list[$i]} * @{$group_list[$j]});
				
				if (!defined $best_merge || ($avg_similarity > $best_merge->{avg_similarity}))
				{
					$best_merge->{avg_similarity} = $avg_similarity;
					$best_merge->{i} = $i;
					$best_merge->{j} = $j;
				}
			}
 		}
 		
 		#Merge the ones we have chosen
 		push @{$group_list[$best_merge->{i}]}, @{$group_list[$best_merge->{j}]};
 		splice @group_list, $best_merge->{j}, 1;
 		
 		#print Dumper($best_merge);
	}
	
	#print Dumper(@group_list);

	#Keep ones that are most diverse from each group
	my @new_sequences;
	foreach (my $i = 0; $i < scalar @group_list; $i++)
	{		
		my $most_different;

		foreach (my $i_index = 0; $i_index < scalar @{$group_list[$i]}; $i_index++)
		{
			my $group_i = $group_list[$i][$i_index];
			my $total_similarity = 0;
			foreach (my $j = 0; $j < scalar @group_list; $j++)
			{				
				foreach my $group_j (@{$group_list[$j]})
				{
					$total_similarity += $matrix[$group_i][$group_j];
				}
			}
			if (!defined $most_different || ($total_similarity < $most_different->{avg_similarity}))
			{
				$most_different->{avg_similarity} = $total_similarity;
				$most_different->{i} = $i_index;
			}		
		}
		#print Dumper($most_different);
		push @new_sequences, $self->seq_ref->[ $group_list[$i][$most_different->{i}] ];
	}
	
	#print +(scalar @new_sequences) . "\n";
	$self->seq(@new_sequences);
	
	#This might have introduced all gapped columns
	$self->delete_gapped_columns;
	
	return 1;
}

=head2 similarity_matrix

 Title   : prune_to_most_diverse
 Usage   : Intergenic::Stockholm->similarity_matrix( $strict );
 Function: Calculate a similarity matrix (as fraction of positions) for the alignment.
 Returns : 
 
=cut
sub similarity_matrix
{
	my ($self, $strict) = @_;

	my $sto = $self->{stockholm};

	my $rf_len = $self->alignment_length;
	my @rf_list;
	if (!$strict)
	{
		$self->throw("Alignment must have an RF line.") if (!$sto->{GC}->{RF});
		my $rf = "\U$sto->{GC}->{RF}";
		@rf_list = split //,  $rf;
		foreach (my $i = 0; $i < scalar @rf_list; $i++)
		{
			$rf_list[$i] = 0 if (! ($rf_list[$i]=~ m/[ATCGU]/) );
		}
		
		$rf =~ s/[^ATCGU]//g;
		$rf_len = length $rf;
	}
	
	my @matrix;
	foreach (my $i = 0; $i < scalar $self->seq; $i++)
	{
		#print "$i\n";
		my $on_seq = $sto->{seq}->[$i]->{seq};
		
		foreach (my $j = 0; $j < $i; $j++)
		{
			my $test_seq = $self->seq_ref->[$j]->{seq};
		
			my $pi = 0;
			foreach (my $q = 0; $q < length $test_seq; $q++)
			{
				next if (!$strict && !$rf_list[$q]);
				$pi++ if ( (substr $on_seq, $q, 1) eq (substr $test_seq, $q, 1) );
			}
			$pi /= $rf_len;
			
			$matrix[$i][$j] = $pi;
			$matrix[$j][$i] = $pi;
		}
		
		$matrix[$i][$i] = 1;
	}
	
	return @matrix;
}

=head2 delete_gapped_columns

 Title   : delete_gapped_columns
 Usage   : Intergenic::Stockholm->delete_gapped_columns();
 Function: Delete columns that are all gaps
 Returns : 
 
=cut
sub delete_gapped_columns
{
	my ($self, $fraction) = @_;

	#Check each column
	#print +($self->alignment_length) . "\n";
	
	my $gap_cutoff = (defined $fraction) ? (1 - $fraction) * scalar $self->seq : 0;
	
	COL: foreach (my $i = 0; $i < $self->alignment_length; $i++)
	{
		my $total_nongap = 0;;
		foreach (my $s = 0; $s < scalar $self->seq; $s++)
		{
			$total_nongap++ if (!gap_char(substr $self->seq_ref->[$s]->{seq}, $i, 1));
			next COL if ($total_nongap >= $gap_cutoff);
			#print +(substr $self->seq_ref->[$s]->{seq}, $i, 1) . " Gap!!\n";
		}

		#Remove it!!
		foreach (my $s = 0; $s < scalar $self->seq; $s++)
		{
			$self->seq_ref->[$s]->{seq} = (substr $self->seq_ref->[$s]->{seq}, 0, $i) . (substr $self->seq_ref->[$s]->{seq}, $i+1);
		}
		
		#Fix secondary structure line
		my $pc = $self->paired_columns();
		foreach my $p (@$pc)
		{
			if ($p->{open} == $i)
			{
				my $ss = $self->ss_cons();
				substr($ss, $p->{close}, 1) = ".";
				$self->ss_cons($ss);
			}
			elsif ($p->{close} == $i)
			{
				my $ss = $self->ss_cons();
				substr($ss, $p->{open}, 1) = ".";
				$self->ss_cons($ss);
			}

		}
		
		#Fix GC lines
		my $sto = $self->{stockholm};
		foreach my $key (keys %{$sto->{GC}})
		{
			$sto->{GC}->{$key} = (substr $sto->{GC}->{$key}, 0, $i) . (substr $sto->{GC}->{$key}, $i+1);
		}
		$i--;
	}
}


sub gap_char
{
	my ($c) = @_;
	return !($_not_gaps =~ m/\Q$c/i);
}

sub alignment_length
{
	my ($self) = @_;
	return length $self->seq_ref->[0]->{seq};
}


sub seq
{
	my ($self, @new_seq) = @_;
	@{$self->{stockholm}->{seq}} = @new_seq if (@new_seq);
	return @{$self->{stockholm}->{seq}};
}

sub seq_ref
{
	my ($self, @new_seq) = @_;
	$self->{stockholm}->{seq} = \@new_seq if (@new_seq);
	return $self->{stockholm}->{seq};
}

sub seq_hash 
{
	my ($self) = @_;
	return $self->{stockholm}->{seq_hash};
}

sub remove_seq
{
	my ($self, $name) = @_;
	
	foreach (my $i = 0; $i < scalar @{$self->seq_ref}; $i++)
	{
		if ( $self->seq_ref->[$i]->{name} eq $name)
		{
			splice @{$self->seq_ref}, $i, 1;
			last;
		}
	} 
	delete $self->seq_hash->{$name};
}

sub add_seq
{
	my ($self, $name, $seq) = @_;
	my $new = { name=>$name, seq=>$seq };
	push @{$self->{stockholm}->{seq}}, $new;
	$self->{stockholm}->{seq_hash}->{$name} = $new;
}

#This is not robust.
sub reverse_complement
{
	my ($self, $name, $seq) = @_;
	
	foreach my $seq ($self->seq())
	{
		
		#Fix name (if in standard format)
		if ($seq->{name} =~ m/(.+)\/(.+)-(.+)/)
		{
			my $accession = $1;
			my $start = $2;
			my $end = $3;
			$seq->{name} = "$accession/$end-$start";
		}
	
		$seq->{seq} = scalar reverse split(//, $seq->{seq});
		$seq->{seq} =~ tr/AUCGaucg/UAGCuagc/;
	}

	my $new_ss = scalar reverse split(//, $self->ss_cons);
	$new_ss =~ tr/<>/></;
	
	$self->ss_cons( $new_ss );
	
}

#This needs error checking!
sub merge
{
	my ($self, $sto) = @_;

	#GF
	foreach my $key ( keys %{$sto->{stockholm}->{GF}} )
	{
		$self->{stockholm}->{GF}->{$key} = $sto->{stockholm}->{GF}->{$key}; 
	}
	
	#GR
	for (my $i = 0; $i < scalar $sto->seq(); $i++)
	{
		foreach my $key (keys %{$sto->seq_ref->[$i]->{GR}} )
		{
			$self->{seq_hash}->{$sto->seq_ref->[$i]->{name}}->{GR}->{$key} = $sto->seq_ref->[$i]->{GR}->{$key};
		}
	}

	#GC
	foreach my $key (keys %{$sto->{stockholm}->{GC}})
	{
		$self->{stockholm}->{GC}->{$key} = $sto->{stockholm}->{GC}->{$key};
	}
	
	#GS
	
	#Write each GS key as a separate chunk

	for (my $i = 0; $i < scalar @{$self->{stockholm}->{seq}}; $i++)
	{
		foreach my $key (keys %{$sto->seq_ref->[$i]->{GS}} )
		{
			$self->{seq_hash}->{$sto->seq_ref->[$i]->{name}}->{GS}->{$key} = $sto->seq_ref->[$i]->{GS}->{$key};
		}
	}
		
}


sub get_column_arrays
{
	my ($self, $prefix) = @_;
	
	my $i = 1;
	my @row_list;
	while ( my $extract =  $self->{stockholm}->{GC}->{"${prefix}_$i"})
	{
		#print STDERR "$extract\n";
		my @pos_list;
		my $on_pos = -1;
		while ( ($on_pos = index ($extract, "*", $on_pos + 1)) != -1)
		{
			print STDERR "$on_pos\n";
			push @pos_list, $on_pos;
		}
		die "Could not find '*' columns in #=GC extract_$i line." if (scalar @pos_list < 1);
		#print STDERR "Extracting positions $start_pos to $end_pos.\n";
		push @row_list, \@pos_list;
		
		$i++;
	}
	
	return @row_list;
}

sub collapse_columns
{
	my ($self, $start, $end) = @_;

	print "$start, $end\n";
	
	my @num_list;
	my $max_digits = 0;
	foreach my $seq ($self->seq())
	{
		#Count the number of non-gaps in the interval
		my $test = substr($seq->{seq}, $start, $end - $start + 1);
		
		my $pattern = "[^$_not_gaps]";
		$test =~ s/$pattern//ig;
		my $num = length $test;
		
	#	$num = '' if ($num == 0);
		push @num_list, $num;

		my $digits = length "$num";
		if ($digits > $max_digits)
		{
			$max_digits = $digits;
		}
		
	}
	
	foreach my $seq ($self->seq())
	{
		my $num = shift @num_list;
		my $replacement;
		if ($num)
		{
			$replacement = sprintf(".%.${max_digits}u.", $num) ;
		}
		else
		{
			$replacement = "." x ($max_digits + 2);
		}
		$replacement =~ s/ /./;
		substr($seq->{seq}, $start, $end - $start + 1) = $replacement;
	}
	
	foreach my $key (sort keys %{$self->{stockholm}->{GC}})
	{
		substr($self->{stockholm}->{GC}->{$key}, $start, $end - $start + 1) = "." x ($max_digits + 2);
	}
	
	return $max_digits + 2;
	
}

return 1;
