#!/usr/bin/perl -w

# Copyright (c) 2005 Jeffrey E. Barrick
# All rights reserved.  Redistribution is not permitted without the 
# express written permission of the authors.

use strict;
	
package Job::Submitter;
use vars qw(@ISA);

use Data::Dumper;

@ISA = qw( );


=head1 NAME

Job::Submitter - control Job submission of a long task list and retrieve results.

=head1 SYNOPSIS

my $xs = Job::Submitter::new(command => 'blastall -p blastp -d cog', local_executable => 1, 
si => ( '/tmp/1.fna', '/tmp/sequence's ) , so => '/tmp/blastresults');

$xs->print();

$xs->submit();

=head1 DESCRIPTION

The job list will be created from the arguments to new as follows:

=over

'host' => Job host

'password' => Job password

'command' => command to be executed (REQUIRED) 'STDOUT' and 'STDIN' in the arguments will be replaced with the proper file name.

'local_executable' => if true, it assumes the executable does not need to be copied.  Instead it creates a script with the command to be sent and executed by Job.

'si' => list of files and or directories to process.  By default all files in a directory (but not subdirectories will be processed).

'so' => directory that will contain output files.

'no_Job' => if set to 1 then Job is not used (this reduces the module to just sequentially doing the task on the local machine which can be helpful for short tasks).

'processors' => maxumum number of jobs to submit at a time

'wait' +> time to wait between making submissionand re-polling

=back

=head1 METHODS

=cut


=head2 new

 Title   : new
 Usage   : $dbcache = Intergenic::Cache->new( -cache_name => 'cog' );
 Function: Creates/opens a database to cache values to
 Returns : a new DatabaseCache object

=cut

our $DB_NAME = "cache";

sub new
{
	my($caller,%args) = @_;
	my $class = ref($caller) || $caller;

	#Set up variables from input and defaults
	my $self = \%args;
	$self->{command} or die "You must provide a command to Job::Submitter";
	bless ($self, $class);
	
	$self->{name} = 'Job_submitter_command' if (!$self->{name});
	$self->{name} =~ s/ /_/g; #No spaces in this name because we use it for script names
	$self->{processors} = 10 if (!$self->{processors});
	$self->{wait} = 10 if (! defined $self->{wait});
	$self->{args} or $self->{args} = '';
	
	#Test the host/password perhaps.	

	#Fix the input file list
	my $input_list_ref = _dir_file_list_to_file_list(@{$self->{si}});
	
	#Remove certain extensions...because we might be restarting an interrupted process
	@$input_list_ref = grep !/$self->{exclude}/, @$input_list_ref if ($self->{exclude});

	#Create the output directory if necessary
	`mkdir $self->{so} >& /dev/null` if (! -e $self->{so});
	
	#Create the job list
	$self->{job_list} = ();
	JOB: foreach my $input (@$input_list_ref)
	{
		my $job;
		$job->{si} = $input;
		
		#Convert to $so
		
		$job->{so} = $job->{si};
		$job->{so} =~ s/.+\///;
		$job->{so} = "$self->{so}/$job->{so}";
		$job->{so} .= "." . $self->{o_ext} if (defined $self->{o_ext});
		
		#if this output file already exists then don't queue the job
		next JOB if (-e $job->{so});
		
		push @{$self->{job_list}}, $job;
	}
	
	
	#Optionally sort the job list
	@{$self->{job_list}} = sort { -s $a->{si} <=> -s $b->{si} } @{$self->{job_list}} if ($self->{sort_by_size});
	
	
	#Create the input script if necessary
	
	return $self;
}


sub print
{
	my($self) = @_;

	print Dumper($self->{job_list});
}

sub submit
{
	my($self) = @_;
	
	
	if ((!defined $self->{job_list}) || (scalar @{$self->{job_list}} == 0) )
	{
		print "There are no jobs to complete.\n";
		return;
	}
	
	$self->submit_local();
}


sub submit_local
{
	my($self) = @_;
	
	foreach my $job (@{$self->{job_list}})
	{
		my $command = "$self->{command} $self->{args}"; 
		
		if ($self->{stdin_arg})
		{
			$command =~ s/STDIN/$job->{si}/g or die "STDIN requested as an argument but not found.";
		}
		else
		{
			$command .=  " < $job->{si}";
		}
		
		if ($self->{stdout_arg})
		{
			$command =~ s/STDOUT/$job->{so}/g or die "STDOUT requested as an argument but not found.";
		}
		else
		{
			$command .=  " > $job->{so}";
		}
		
		my $result = system $command;
		
		if ($result) #There was an error of some sort
		{
			#We need to remove the output file or we might believe it was done
			`rm $job->{so}`;
			die "Error returned: $result\n$command\n";
		}
	}
}


sub _dir_file_list_to_file_list
{
	my @file_list;
		
	foreach my $file_or_dir (@_)
	{
		if (-d $file_or_dir)
		{
			opendir(INDIR, $file_or_dir) or die "Cannot open directory: $file_or_dir";
			my @dir_files = grep !/^\./, readdir INDIR;
			@dir_files = grep !/^_/, @dir_files;
			
			foreach my $file ( @dir_files)
			{
				push @file_list, "$file_or_dir/$file" if (! -d "$file_or_dir/$file");
			}
		}
		else
		{
			push @file_list, $file_or_dir;
		}
	}
	return \@file_list; 
}


return 1;