#!/usr/bin/perl


#This program reads in stockholm alignments with weights and out puts the mutual information in stockholm format.  The -m allows for a mutual information
#cutoff vaule to be input.  If the -b option is used it causes base pairs and non base pairs to be sorted into two separate lists
use strict;
use FindBin;
use lib $FindBin::Bin;

use Intergenic::Stockholm;

###
# Pod Documentation
###

=head1 NAME

mutual_information.perl

=head1 SYNOPSIS

mutual_information.perl -i /path/to/stockholm/alignment -o /path/to/stockholm/output [-m 0.2 -abrg]

Calculate the mutual information between columns in an alignment.

=head1 DESCRIPTION

=over

=item B<-i> <file path> 
	
Stockholm alignment with weights.

=item B<-o> <file path> 

Stockholm file with mutual information scores appended as #GC lines.

=item B<-m> <mutual information cutoff value>

=item B<-b|base_pair_separator>  <b=0 sorted by mutual information high to low;
			 b=1 base pairing and non base pairing in separate list sorted by mutual information high to low>

=item B<-a|all>  

Show all matches above the mutual information cutoff.
	
=item B<-t|triples> 

Predict base triples, as mutual information between annotated pairs and single bases.

=item B<-r|raw> 

Print output in 'raw' format: an ordered list of numbers.

=item B<-f|full-precentages> 

Print out the full observed percentages of each pair.

=back

=head1 AUTHORS

Jeff Barrick
Beth Grove

=head1 COPYRIGHT

Copyright 2005.  All rights reserved.

=cut

###
# End Pod Documentation
###


use Getopt::Long;
use Pod::Usage;
my ($help, $man);
my ($input_file_name, $output_file_name, $score_cutoff, $renormalized);
my ($base_pair_separator, $all, $raw_file, $full_percentages, $n_best, $no_weights, $weights, $ignore_gaps, $required_percentage);

pod2usage(1) if (scalar @ARGV == 0);

GetOptions(
	'help|?' => \$help, 'man' => \$man,
	'input|i=s' => \$input_file_name,
	'output|o=s' => \$output_file_name,
	'score_cutoff|s=s' => \$score_cutoff,
	'renormalized|z' => \$renormalized,
	'base_pair_separator|b' => \$base_pair_separator,
	'all|a' => \$all,
	'raw|r=s' => \$raw_file,
	'full-percentages|f' => \$full_percentages,
	'n-best|n' => \$n_best,
	'weights|w' => \$weights,
	'required-percentage|p=f' => \$required_percentage,
	'ignore_gaps|g' => \$ignore_gaps,
) or pod2usage(2);

pod2usage(1) if $help;
pod2usage(-exitstatus => 0, -verbose => 2) if $man;
pod2usage(1) if (!defined $input_file_name);

die "Can not have raw output and renormalized scores." if ($renormalized && $raw_file);

$no_weights = not $weights;

$score_cutoff = ($renormalized) ? 0.3 : 0 if (!defined $score_cutoff);
$n_best = 5 if (!$n_best);
 
print "$input_file_name\n";
print "Column:\n";

my $sto = new Intergenic::Stockholm( -filename=>$input_file_name );
my $pair_list_ref = $sto->paired_columns();
my @column_paired_list = $sto->column_paired_array();


open RAW, ">$raw_file" if ($raw_file);

#This gets each of the weight values which was calcuated for each of the sequences using the weight program found in ssh bliss
#and puts them into an array

my @weight_list;
my $sum_weights = 0;

if (!$no_weights)
{
	foreach my $weight (@{$sto->{stockholm}->{seq}})
	{
		die "Input stockholm file is missing weights. " if (! defined $weight->{GS}->{WT});
		push @weight_list, $weight->{GS}->{WT};
		$sum_weights += $weight->{GS}->{WT};
	}
}
#This adds up all of the weights for all of the seqences so that count totals can be divided by the total weight of the
#sequences so that the count values can be normalized

my @seq_list;
foreach my $seq ($sto->seq)
{
	push @seq_list, $seq->{seq};	
} 


my $seq;
my @list;
my @seq_array;

foreach my $seq (@seq_list)
{
	$seq =~ tr /-/\./;
	$seq = "\U$seq";
	my @list=split (//, $seq);
	push @seq_array, \@list;
}

my $number_of_sequences = scalar @seq_array;
$sum_weights = $number_of_sequences if ($no_weights);
my $length_of_sequences = length $seq_list[0];

my @base_list=("A","C","G","U");
push @base_list, '.' if (!$ignore_gaps);

my $base_str = join "", @base_list;


#If we are not ignoring column pairs where a gap occurs in either one, then we can pre-calculate
#the single column informations.
my @Hx;

if (!$ignore_gaps)
{
	#Calculate the single column H scores: H(x) and H(y)

	for (my $x = 0; $x < $length_of_sequences; $x++)
	{

		my %column_counts;
		for(my $s = 0; $s < scalar @seq_list; $s++)
		{		
			$column_counts{$seq_array[$s]->[$x]} += (($no_weights) ? 1 : $weight_list[$s]);
		}
		
		$Hx[$x] = 0;
		foreach my $key (keys %column_counts)
		{
			my $fr = $column_counts{$key} / $sum_weights;
			$Hx[$x] -= $fr * log $fr;
		}
		
		#print "$x " . (join "-", keys %column_counts) . "\n";
	}	
}

#Calculate the double column H scores: H(x,y)

my @Hxy;
my @Mxy;
my @Rxy1;
my @mi;
X: for (my $x = 0; $x < $length_of_sequences; $x++)
{
	print " $x\n";
	Y: for (my $y = $x + 1; $y < $length_of_sequences; $y++)
	{
		my %pairs_count;

		my $occupied_sum_weights = 0;
		
		#Only used if ignoring gaps
		my %column_counts_x;
		my %column_counts_y;
		my $gapped_sum_weights = ($ignore_gaps) ? 0 : $sum_weights;
		###
		#We must recalculate the total weight of the columns we are looking at
		
		
		SEQUENCE: for (my $s = 0; $s < scalar @seq_list; $s++)
		{	
			if ($ignore_gaps)
			{
				next SEQUENCE if ( ($seq_array[$s]->[$x] eq '.') || ($seq_array[$s]->[$y] eq '.') );
				$column_counts_x{$seq_array[$s]->[$x]} += (($no_weights) ? 1 : $weight_list[$s]);
				$column_counts_y{$seq_array[$s]->[$y]} += (($no_weights) ? 1 : $weight_list[$s]);
				$gapped_sum_weights += (($no_weights) ? 1 : $weight_list[$s]);
			}
			
			$pairs_count{ $seq_array[$s]->[$x] . $seq_array[$s]->[$y] } += (($no_weights) ? 1 : $weight_list[$s]);
		}
		
		$Hxy[$x][$y] = 0;
		foreach my $key (keys %pairs_count)
		{
			my $fr = $pairs_count{$key} / $gapped_sum_weights;
			$Hxy[$x][$y] -= $fr * log $fr;
		}
		
		if ($ignore_gaps)
		{
			my $Hx = 0;
			foreach my $key (keys %column_counts_x)
			{
				my $fr = $column_counts_x{$key} / $gapped_sum_weights;
				$Hx -= $fr * log $fr;
			}
			
			my $Hy = 0;
			foreach my $key (keys %column_counts_y)
			{
				my $fr = $column_counts_y{$key} / $gapped_sum_weights;
				$Hy -= $fr * log $fr;
			}
			
			$Mxy[$x][$y] = $Hx + $Hy - $Hxy[$x][$y];
		}
		else
		{
			$Mxy[$x][$y] = $Hx[$x] + $Hx[$y] - $Hxy[$x][$y];
		}
		
		#Prevent other record-keeping if in raw mode
		if ($raw_file)
		{
			printf RAW "%.8e\n", $Mxy[$x][$y];
			next Y;
		}

		#Don't keep data from columns with a certain number of non-gaps
		next Y if ($required_percentage && ($gapped_sum_weights/$sum_weights < $required_percentage));

		#Add an item to the data array		
		
		my $item = { 'c1'=> $x, 'c2'=> $y, 'summary'=> '', 'paired'=>0, 'wc'=>0 };
		
		#Calculate fraction that is Watson-Crick
		$item->{wc} += $pairs_count{"AU"} if ($pairs_count{"AU"});
		$item->{wc} += $pairs_count{"CG"} if ($pairs_count{"CG"});
		$item->{wc} += $pairs_count{"GC"} if ($pairs_count{"GC"});
		$item->{wc} += $pairs_count{"UA"} if ($pairs_count{"UA"});
		$item->{wc} += $pairs_count{"GU"} if ($pairs_count{"GU"});
		$item->{wc} += $pairs_count{"UG"} if ($pairs_count{"UG"});
		$item->{wc} /= $sum_weights if ($gapped_sum_weights);

		#Are these columns paired?
		for (my $number=0; $number < scalar @$pair_list_ref; $number++)
		{
			if ($pair_list_ref->[$number]{"open"}==$x and
			    $pair_list_ref->[$number]{"close"}==$y)
			{
				$item->{paired} = 1;
				last;
			}
		}
		
		my @key_list = sort { -($pairs_count{$a} <=> $pairs_count{$b}) } keys %pairs_count;
		foreach my $key (@key_list)
		{
			$pairs_count{$key} *= 100; #percentage
			$pairs_count{$key} /= $sum_weights if ($gapped_sum_weights);
			
			$item->{summary} .= "," if ($item->{summary});
			$item->{summary} .= sprintf "$key=%.1f%%", $pairs_count{$key};
		}
		
		if ($renormalized)
		{
			my $score_xy = ($Hx[$x] != 0) ? $Mxy[$x][$y] / $Hx[$x] : 0;
			my $item_xy = {'c1'=> $x, 'c2'=> $y, 'score'=> $score_xy, 'summary'=>$item->{summary}, 'paired'=>$item->{paired}, 'wc'=>$item->{wc}};
			
			my $score_yx = ($Hx[$y] != 0) ? $Mxy[$x][$y] / $Hx[$y] : 0;
			my $item_yx = {'c1'=> $y, 'c2'=> $x, 'score'=> $score_yx, 'summary'=>$item->{summary}, 'paired'=>$item->{paired}, 'wc'=>$item->{wc}};
		
			#print "$x $y $Hxy[$x][$y] $Hx[$x] $Hx[$y] $Mxy[$x][$y] $Rxy[$x][$y] \n";
			
			$mi[$x]->[$y] = $item_xy;
			$mi[$y]->[$x] = $item_yx;			
		}
		else
		{
			$item->{'score'} = $Mxy[$x][$y];
			push @mi, $item if ($item->{score} >= $score_cutoff);
		}
	}
}


#Done printing if in raw mode.
exit 0 if ($raw_file);

#R value output
if ($renormalized)
{
	my $counter=1;
	my $end_index = ($all) ? scalar @mi : $n_best;
	LIST: foreach (my $i = 0; $i < scalar @mi; $i++)
	{
		my @mi_list = @{$mi[$i]};
		@mi_list = sort {-($a->{score} <=> $b->{score}) } @mi_list;
		foreach (my $j = 0; $j < $end_index; $j++)
		{	
			my $item = $mi_list[$j];
			next LIST if ($item->{score} < $score_cutoff);

			#Value has polarity from column $i to $j;	
			my $period_string;	
			if ($item->{c1} < $item->{c2})
			{
				$period_string = ('.' x $item->{c1}) . '>' . ('.' x ($item->{c2} - $item->{c1} - 1)) . '*' . ('.' x ($sto->alignment_length - $item->{c2} - 1));
			}
			else
			{
				$period_string = ('.' x $item->{c2}) . '*' . ('.' x ($item->{c1} - $item->{c2} - 1)) . '<' . ('.' x ($sto->alignment_length - $item->{c1} - 1));
			}

			my $name = "T_";
			$name = "U_" if ($base_pair_separator && $item->{paired});
			$name .= sprintf "%06g,MI=%.8e,bp=%.2f(%s)", $counter, $item->{score}, $item->{wc}, ($item->{paired} ? "yes" : "no");
			$name .= ",$item->{summary}" if ($full_percentages);

			$sto->{stockholm}->{GC}->{$name}=$period_string;
			$counter++;
		}
	}
}
else #Normal MI output
{

	@mi = sort {-($a->{score} <=> $b->{score}) } @mi;

	my $counter=1;
	my @printed_columns;
	foreach my $item (@mi) 
	{	
		#last if ($item->{score} <= 0.000001); #No reason to keep these.
		next if (!$all && (($printed_columns[$item->{c1}] > $n_best) && ($printed_columns[$item->{c2}] > $n_best)));
		$printed_columns[$item->{c1}]++;
		$printed_columns[$item->{c2}]++;

		my $period_string = ('.' x $item->{c1}) . '*' . ('.' x ($item->{c2} - $item->{c1} - 1)) . '*' . ('.' x ( length($seq_list[0]) - $item->{c2} - 1));
		my $name = "T_";
		$name = "U_" if ($base_pair_separator && $item->{paired});
		
		#my $print_counter = sprintf("%06g", $counter);		
		#$name .= sprintf "%06g,MI=%.2f,bp=%.2f(%s),%s", $counter, $item->{score}, $item->{wc}, ($item->{paired} ? "yes" : "no"), $item->{summary};
		$name .= sprintf "%06g,MI=%.8e,bp=%.2f(%s)", $counter, $item->{score}, $item->{wc}, ($item->{paired} ? "yes" : "no");

		$name .= ",$item->{summary}" if ($full_percentages);

		$sto->{stockholm}->{GC}->{$name}=$period_string;
		$counter++;
	}
}


$sto->writefile( $output_file_name );



