#!/usr/bin/perl -w

###
# Pod Documentation
###

=head1 NAME

mutual_information_significance.pl

=head1 SYNOPSIS

Usage: mutual_information_significance.pl -i stockholm.stk -o stockholm.mi.stk [-r 1000 -n 1000]

Calculate the significance of mutual information scores between aligment columns in a Stockholm alignment
to examine support for base interactions.

=head1 DESCRIPTION

=over

=item B<-i> <file path> 

Input Stockholm file.

=item B<-o> <file path> 

Output Stockholm file with column pairs with significant of mutual information shown.

=item B<-r> <integer> 

The number of resampled alignments to create. The greater the number the more accurate 
the estimated p-values for each pair will be.  Use 1000 to 10,000 for publication quality. Be warned that
high values consume huge amounts of disk space! DEFAULT = 200.

=item B<-n> <integer>

The actual number of sequences to use in calculations.  If more sequences
are input then the alignment is pruned to the most diverse sequences. DEFAULT = 300.

=back

=head1 AUTHOR

Jeffrey Barrick

=head1 COPYRIGHT

Copyright 2005.  All rights reserved.

=cut

###
# End Pod Documentation
###

use strict;

use FindBin;
use lib $FindBin::Bin;
$ENV{PATH} = "$ENV{PATH}:" . $FindBin::Bin;

use Job::Submitter;
use Intergenic::Stockholm;

#Set these parameters


#Get options
use Getopt::Long;
use Pod::Usage;
my ($help, $man);
my ($mi_path, $tree_path, $stockholm_file, $output_file, $number, $resamplings, $fraction_identity);
#pod2usage(1) if (scalar @ARGV == 0);
GetOptions(
	'help|?' => \$help, 'man' => \$man,
	'mutual-information-path|m=s' => \$mi_path,
	'tree-path|t=s' => \$tree_path,
	'input|i=s' => \$stockholm_file,
	'output|o=s' => \$output_file,
	'number|n=s' => \$number,
	'resamplings|r=s' => \$resamplings,
) or pod2usage(2);
pod2usage(1) if $help;
pod2usage(-exitstatus => 0, -verbose => 2) if $man;
#pod2usage(1) if (!defined $covariation_path || !defined $tree_path || !defined $stockholm_file || !defined $output_file);
pod2usage(1) if (!defined $output_file);

my $mi_options  = "-g -f 0.5";
die "Input and output file names must be different.\n" if ($output_file eq $stockholm_file);
die "Output file already exists.\n" if (-e $output_file);

#default parameters are directories in the current path
$tree_path = "resampled-tree" if (!defined $tree_path);
$mi_path = "resampled-mi" if (!defined $mi_path);
$fraction_identity = 1.0 if (!defined $fraction_identity);
$number = 300 if (!defined $number);
$resamplings = 200 if (!defined $resamplings);

##
# Prune the input stockholm file (if necessary)
##

my $pruned_stockholm_file = $stockholm_file . ".pruned";
if (!-e "$pruned_stockholm_file")
{
	my $percent = sprintf("%.1f", $fraction_identity * 100);
	print STDERR "Pruning alignment to at most $number sequences with <$percent\% identity at reference positions...\n";
	my $sto = new Intergenic::Stockholm( -in => $stockholm_file );
	my $num_seq = scalar $sto->seq;
	my $aln_length = $sto->alignment_length;
	print STDERR "Initial alignment has $num_seq sequences and $aln_length columns.\n";

	$sto->remove_redundant($fraction_identity, "strict");
	$sto->prune_to_most_diverse($number, "strict");
	
	my $gap_fraction = 0.5;
	my $gap_percent = sprintf("%.1f", $gap_fraction * 100);
	print STDERR "Removing columns with >$gap_percent\% gaps...\n";
	$sto->delete_gapped_columns(0.5);
	$sto->writefile("$pruned_stockholm_file");
	
	$num_seq = scalar $sto->seq;
	$aln_length = $sto->alignment_length;
	print STDERR "Pruned alignment has $num_seq sequences and $aln_length columns.\n";

}
$stockholm_file = $pruned_stockholm_file;

##
# Calculate mutual information for the initial file
##
my $stockholm_wt_file = $stockholm_file . ".wt";
my $stockholm_mi_file = $stockholm_file . ".mi";


if (!-e $stockholm_wt_file)
{
	my $err = system "esl-weight -g --rna $stockholm_file > $stockholm_wt_file";
	if ($err)
	{
		`rm $stockholm_wt_file` if (-e $stockholm_wt_file);
		die "Error $err calling 'esl-weight' (from the INFERNAL package)";
	}
}

if (!-e $stockholm_mi_file)
{
	my $err = system "mutual_information.pl -f -i $stockholm_wt_file -o $stockholm_mi_file $mi_options";
	die "Error $err calling 'mutual_information.pl'" if ($err);
	
	if ($err)
	{
		`rm $stockholm_mi_file` if (-e $stockholm_mi_file);
		die "Error $err calling 'mutual_information.pl'";
	}
}

##
# Calculate the tree with rate4site
##
my $phylip_file = $stockholm_file . ".phylip";

#Convert to phylip format...
if (!-e $phylip_file)
{
	my $err = system "stockholm_to_phylip.pl -p -i $stockholm_file -o $phylip_file";
	die "Error $err calling 'stockholm_to_phylip.pl'" if ($err);
	
	if ($err)
	{
		`rm $phylip_file` if (-e $phylip_file);
		die "Error $err calling 'stockholm_to_phylip.pl'";
	}
}

my $tree_file = $stockholm_file . ".tree";
my $urates_file = $stockholm_file . ".urates";
my $rates_file = $stockholm_file . ".rates";

if (!-e $urates_file || !-e $tree_file)
{
	my $err = system "rate4site -a all -k 50 -mn -zj -ib -s $phylip_file -x $tree_file -y $urates_file -o $rates_file";
	if ($err)
	{
		`rm $tree_file` if (-e $tree_file);
		`rm $urates_file` if (-e $urates_file);
		`rm $rates_file` if (-e $rates_file);
		die "Error $err calling 'rate4site'";
	}

}


##
# Simulate the requested number of trees
##

if (!-e $tree_path) 
{
	my $err = system "simulate_tree.pl -i $tree_file -r $urates_file -s $stockholm_wt_file -o $tree_path -n $resamplings";
	
	if ($err)
	{
		`rm -r $tree_path` if (-e $tree_file);
		die "Error $err calling 'simulate_tree.pl'";
	}	
}

##
# Batch calculate the mutual information outputs for each resampled alignment
##

my $mutual_info_xs = new Job::Submitter( 
				name => 'covariation',
				stdin_arg => 1,
				stdout_arg => 1,
				command => "mutual_information.pl", 
				args => "-r STDOUT -i STDIN $mi_options",
				si => [$tree_path], 
				so => $mi_path,
				no_xgrid => 1
				);

#$mutual_info_xs->print();
$mutual_info_xs->submit();

##
# Calculate the significances of the true mutual information scores by reading these files
##

my $sto = new Intergenic::Stockholm( -in => $stockholm_mi_file );

my $track_hash;

#determine what columns we are going to compare.
my @sig_list;
foreach my $gc (keys %{$sto->{stockholm}->{GC}})
{
	if ( $gc =~ m/MI=(.+?),/)
	{
		my $item;
		$item->{mi} = $1;
		$item->{key} = $gc;
		$item->{value} = $sto->{stockholm}->{GC}->{$gc};
		$item->{p} = 0;
		#recover the column numbers with pattern matching
		if ($sto->{stockholm}->{GC}->{$gc} =~ m/\*/g)
		{
			$item->{c1} = pos($sto->{stockholm}->{GC}->{$gc})-1;
			if ($sto->{stockholm}->{GC}->{$gc} =~ m/\G.*\*/g)
			{
				$item->{c2} = pos($sto->{stockholm}->{GC}->{$gc})-1;
			}
		}
		
		#Define that we are paying attention to these columns
#		push @{$track_hash->{$item->{c1}}}, $item;
#		push @{$track_hash->{$item->{c2}}}, $item;
		$track_hash->{"$item->{c1}-$item->{c2}"} = $item;
		
		push @sig_list, $item;
	}
}


my $mi_file_list = Job::Submitter::_dir_file_list_to_file_list($mi_path);
my $c = 0;
foreach my $mi_file (@$mi_file_list)
{
	$c++;
	print "File $c\n";
	open SIG, "$mi_file";
	for(my $i=0; $i<$sto->alignment_length; $i++)
	{
		for(my $j=$i+1; $j<$sto->alignment_length; $j++)
		{
			my $line = <SIG>;
			chomp $line;
			
			#foreach my $k ($i, $j)
			#{
				my $test = $track_hash->{"$i-$j"};
				next if (!defined $test);
								
			#	foreach my $test (@{$track_hash->{$k}})
			#	{
					if ($line >= $test->{mi})
					{
						$test->{p} += ($line == $test->{mi}) ? 0.5 : 1;
					}
			#	}
			#}	
		}
	}
}


#The number of MI values compared
my $n = scalar(@$mi_file_list);# * ($sto->length - 1);

foreach my $sig (@sig_list)
{
	#print "$sig->{p}\n";
	#$sig->{p} = scalar keys %{$sig->{p}};
	$sig->{p} /= $n;
}

#Sort
@sig_list = sort { ($a->{p}  <=> $b->{p}) || -($a->{mi} <=> $b->{mi}) } @sig_list;

my $counter = 1;	
foreach my $sig (@sig_list)
{
	delete $sto->{stockholm}->{GC}->{$sig->{key}};
	next if ($sig->{p} > 0.2);

	#Add the information to the Stockholm file
	
	my $name = "T_";
	$name = sprintf "T_%06g", $counter++;
	
	$sig->{key} =~ s/(.+?),/$name,/;	
	$sig->{key} .= ",p=$sig->{p}($n)";
	
	#Fix the key so it will be sorted correctly
	$sto->{stockholm}->{GC}->{$sig->{key}} = $sig->{value};
	
	#print "$sig->{key}\n";
}


$sto->writefile($output_file);
