#!/usr/bin/perl -w

###
# Pod Documentation
###

=head1 NAME

simulate_tree.perl

=head1 SYNOPSIS

Usage: simulate_tree.perl -i <path/to/tree/file> -r <path/to/unnormalized/rate/file> -s <path/to/original/stockholm/file -o <path/to/output/stockholm/files> -n 100

Reads a rate4site tree and rates file to simulate evolution from a fake ancestor sequence (default, poly A).

=head1 DESCRIPTION

=over

=item B<-i> <file path> REQUIRED

Path to Phylip format tree file output by rate4site. Must have numbers for nodes that match the Stockholm file!

=item B<-r> <file path> REQUIRED

Path to B<unnormalized> rate file output by rate4site.

=item B<-s> <file path> REQUIRED

Original Stockholm file (to be used as a template for re-inserting gaps and having structure).

=item B<-o> <directory path> REQUIRED

Path where new alignments will be created.

=item B<-n> <integer> 

Number of new alignments to generate. (Default = 100)

=item B<-s> <integer> 

Number of to start with when making filenames of new alignments.

=back

=head1 AUTHOR

Jeffrey Barrick

=head1 COPYRIGHT

Copyright 2005.  All rights reserved.

=cut

###
# End Pod Documentation
###

use strict;

use FindBin;
use lib $FindBin::Bin;
use POSIX;
use Data::Dumper;

#Get options
use Getopt::Long;
use Pod::Usage;
my ($help, $man);
my ($tree_file, $rates_file, $stockholm_file, $output_path, $n, $s);
#pod2usage(1) if (scalar @ARGV == 0);
GetOptions(
	'help|?' => \$help, 'man' => \$man,
	'in|i=s' => \$tree_file,
	'rates|r=s' => \$rates_file,
	'stockholm|s=s' => \$stockholm_file,
	'out|o=s' => \$output_path,
	'number|n=s' => \$n,
	'start|1=s' => \$s,
) or pod2usage(2);
pod2usage(1) if $help;
pod2usage(-exitstatus => 0, -verbose => 2) if $man;
pod2usage(1) if (!defined $tree_file || !defined $rates_file || !defined $stockholm_file || !defined $output_path);
$s = 1 if (!defined $s);

#Read the tree
use Bio::TreeIO;
my $treeio = new Bio::TreeIO('-file' => $tree_file);
my $tree = $treeio->next_tree;


#Read the rates file into a list of rates
open RATES, "$rates_file";
my @rates_lines = <RATES>;
chomp @rates_lines;
@rates_lines = grep /^\s*[^#]/, @rates_lines;
close RATES;

my @rate_list;
foreach my $rates_line (@rates_lines)
{
	$rates_line =~ m/^\s*(\d+)\s+\S+\s+(\S+)/;
	$rate_list[$1 - 1] = $2;
}


my $base_hash = {'A'=>0, 'U'=>1, 'C'=>2, 'G'=>3 };
my $base_num = 4;
my @base_list = ('A', 'U', 'C', 'G');


use Intergenic::Stockholm;
my $sto = new Intergenic::Stockholm(-in => "$stockholm_file");

mkdir "$output_path" if (! -e "$output_path");

#Generate trees
foreach (my $i = $s; $i < $s + $n; $i++)
{
	#Start at the root and recurse through nodes

	my @seq_list;
	my $seq; 
	@$seq= split //, ('A' x $sto->alignment_length);
	SimulateEvolution($tree->get_root_node(), \@seq_list, $seq);

	#Add back gaps from the original stockholm file
	my $new_sto = new Intergenic::Stockholm(-in => "$stockholm_file");
	for (my $j = 0; $j < scalar $new_sto->seq; $j++) 
	{
		my @split_ref_seq = split //, $new_sto->seq_ref()->[$j]->{seq};
		
		foreach (my $k = 0; $k < scalar @{$seq_list[$j]}; $k++)
		{
			$seq_list[$j][$k] = "." if ( "-." =~ m/\Q$split_ref_seq[$k]/ );
		}
	
		$new_sto->seq_ref()->[$j]->{seq} = join "", @{$seq_list[$j]};
	}
	$new_sto->writefile("$output_path/$i.sto");	
}




sub SimulateEvolution
{
	my ($node, $seq_list, $seq) = @_;
	
	#Evolve the sequence _to_ this node from the acenstor that was passed.
	my $bl = $node->branch_length;
	$bl = 0 if (!defined $bl);
	
	my $new_seq;
	for ( my $i = 0; $i < scalar @$seq; $i++ )
	{
		push @$new_seq, JCMutateBase( $seq->[$i], $bl * $rate_list[$i] ), ;
	}
	
	if ($node->is_Leaf)
	{
		$seq_list->[$node->id-1] = $new_seq;
	}
	
	
	foreach my $child ($node->each_Descendent)
	{
		SimulateEvolution($child, $seq_list, $new_seq);
	}
	
}



sub JCMutateBase
{
	my ($base, $ut) = @_;
	
	my $pr_change = (1 - (exp -$ut)) * 0.25;
	my $random = rand;
	
	#no change
	return $base if ($random >= 3 * $pr_change);
	
	#increment the base
	my $bn = $base_hash->{$base} + 1 + int ($random / $pr_change);
	return $base_list[$bn % $base_num];
	
}


