#!/usr/bin/perl -w

###
# Pod Documentation
###

=head1 NAME

stockholm_to_phylip.perl

=head1 SYNOPSIS

Usage: stockholm_to_phylip.perl -i <path/to/input/stockholm/file> -o <path/to/output/stockholm/file> [-p]

Convert Stockholm format to phylip input format.

=head1 DESCRIPTION

=over

=item B<-i> <file path> 

Input Stockholm file.

=item B<-o> <file path> 

Output phylip input file.

=item B<-p>

Preserve all columns (otherwise >10% gapped are removed)

=back

=head1 AUTHOR

Jeffrey Barrick

=head1 COPYRIGHT

Copyright 2005.  All rights reserved.

=cut

###
# End Pod Documentation
###

use strict;

use FindBin;
use lib $FindBin::Bin;

use Intergenic::Stockholm;

#Get options
use Getopt::Long;
use Pod::Usage;
my ($help, $man);
my ($input_file, $output_file, $preserve_gaps);
#pod2usage(1) if (scalar @ARGV == 0);
GetOptions(
	'help|?' => \$help, 'man' => \$man,
	'in|i=s' => \$input_file,
	'out|o=s' => \$output_file,
	'preserve-gaps|p' => \$preserve_gaps,
) or pod2usage(2);
pod2usage(1) if $help;
pod2usage(-exitstatus => 0, -verbose => 2) if $man;
pod2usage(1) if (!defined $input_file || !defined $output_file);


my $sto = new Intergenic::Stockholm(-in => "$input_file");
#$sto->remove_redundant(1.0); use prune!
my $name_width =10;

#Fix alignment format
foreach my $seq (@{$sto->{stockholm}->{seq}} ) 
{
	$seq->{seq} = "\U$seq->{seq}";
	$seq->{seq} =~ tr/\.U/-T/;
}

if (!$preserve_gaps)
{
	my $total_weight;
	foreach my $seq (@{$sto->{stockholm}->{seq}} ) 
	{
		die "Input alignment must have weights!" if (!defined $seq->{GS}->{WT});
		$total_weight += $seq->{GS}->{WT};
	}

	#Remove columns with more than 10% gaps
	foreach (my $i = 0; $i < $sto->length; $i++)
	{
		my $gap_weight = 0;
		foreach my $seq (@{$sto->{stockholm}->{seq}} ) 
		{
			#print $i . " " . $seq->{seq} . "\n";
			$gap_weight += $seq->{GS}->{WT} if ("-" eq (substr $seq->{seq}, $i, 1));
		}

		if ($gap_weight / $total_weight > 0.10)
		{
			foreach my $seq (@{$sto->{stockholm}->{seq}} ) 
			{
				substr $seq->{seq}, $i, 1, "";
			}
			$i--;
		}
	}
}

=comment

#Check to see if there are any all gap columns
foreach (my $i = 0; $i < $sto->length; $i++)
{
	my $found = 0;
	SEQ: foreach my $seq ( $sto->seq() ) 
	{
		if  ("-" eq (substr $seq->{seq}, $i, 1))
		{
			$found++;
			last SEQ;
		}
	}
	die "Problem with column $i" if (!$found);
}

=cut

open OUT, ">$output_file";
print OUT scalar @{$sto->{stockholm}->{seq}} . " " . (length $sto->{stockholm}->{seq}->[0]->{seq}) . "\n";

my $i = 1;
foreach my $seq (@{$sto->{stockholm}->{seq}})
{
	printf OUT "\%-${name_width}.${name_width}s%s\n", $i, $seq->{seq};	
	$i++;
}
